#
# spec file for package lua
#
# Copyright (c) 2014 SUSE LINUX Products GmbH, Nuernberg, Germany.
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via http://bugs.opensuse.org/
#


Name:           lua
Version:        5.2.2
Release:        4.5
Summary:        Small Embeddable Language with Simple Procedural Syntax
License:        MIT
Group:          System/Libraries
Url:            http://www.lua.org
Source:         http://www.lua.org/ftp/lua-%{version}.tar.gz
Source1:        macros.lua
Source2:        baselibs.conf
# prefix and build shared libraries (upstream don't want this in Makefile)
# visibility: I don't see the point why divert from upstream except that
#             luaU_dump, luaP_opmodes and luaP_opnames need to be visible
#             to luac; if you will divert, don't forgot that LUAI_DDEC
#             prefixes declaration of _variables_
Patch0:         %{name}-suse.diff
Patch1:         lua-CVE-2014-5461.patch
%define major_version 5.2
%define libsuf        5_2
BuildRequires:  pkg-config
BuildRequires:  readline-devel
BuildRequires:  update-alternatives
BuildRoot:      %{_tmppath}/%{name}-%{version}-build
Requires(pre):  update-alternatives

%description
Lua is a programming language originally designed for extending
applications, but also frequently used as a general-purpose,
stand-alone language.

Lua combines simple procedural syntax (similar to Pascal) with powerful
data description constructs based on associative arrays and extensible
semantics. Lua is dynamically typed, interpreted from byte codes, and
has automatic memory management, making it ideal for configuration,
scripting, and rapid prototyping. Lua is implemented as a small library
of C functions, written in ANSI C, and the implementation goals are
simplicity, efficiency, portability, and low embedding cost.

%package devel
Summary:        Development files for lua
Group:          Development/Libraries/C and C++
Requires:       %{name} = %{version}
Requires:       liblua%{libsuf} = %{version}
Conflicts:      lua50-devel
Conflicts:      lua51-devel

%description devel
Lua is a programming language originally designed for extending
applications, but also frequently used as a general-purpose,
stand-alone language.

This package contains files needed for embedding lua into your
application.

%package -n liblua%{libsuf}
Summary:        Small Embeddable Language with Simple Procedural Syntax
Group:          System/Libraries
Provides:       %{name}-libs = %{version}
Obsoletes:      %{name}-libs < %{version}

%description -n liblua%{libsuf}
Lua is a programming language originally designed for extending
applications, but also frequently used as a general-purpose,
stand-alone language.

Lua combines simple procedural syntax (similar to Pascal) with powerful
data description constructs based on associative arrays and extensible
semantics. Lua is dynamically typed, interpreted from byte codes, and
has automatic memory management, making it ideal for configuration,
scripting, and rapid prototyping. Lua is implemented as a small library
of C functions, written in ANSI C, and the implementation goals are
simplicity, efficiency, portability, and low embedding cost.

%package doc
Summary:        Small Embeddable Language with Simple Procedural Syntax
Group:          System/Libraries
%if 0%{?suse_version} >= 1120
BuildArch:      noarch
%endif

%description doc
Lua is a programming language originally designed for extending
applications, but also frequently used as a general-purpose,
stand-alone language.

Lua combines simple procedural syntax (similar to Pascal) with powerful
data description constructs based on associative arrays and extensible
semantics. Lua is dynamically typed, interpreted from byte codes, and
has automatic memory management, making it ideal for configuration,
scripting, and rapid prototyping. Lua is implemented as a small library
of C functions, written in ANSI C, and the implementation goals are
simplicity, efficiency, portability, and low embedding cost.

%prep
%setup -q
%patch0 -p1
%patch1

%build
sed -i -e "s@lib/lua/@%{_lib}/lua/@g" src/luaconf.h
make %{?_smp_mflags} -C src CC="gcc" MYCFLAGS="%{optflags} -std=gnu99 -D_GNU_SOURCE -fPIC -DLUA_USE_LINUX" MYLIBS="-Wl,-E -ldl -lreadline -lhistory -lncurses" V=%{major_version} all

%install
make install INSTALL_TOP="%{buildroot}%{_prefix}" INSTALL_LIB="%{buildroot}%{_libdir}" INSTALL_CMOD=%{buildroot}%{_libdir}/lua/%{major_version} INSTALL_MAN="%{buildroot}%{_mandir}/man1"
for file in lua luac ; do
    mv "%{buildroot}%{_bindir}/${file}"        "%{buildroot}%{_bindir}/${file}%{major_version}"
    mv "%{buildroot}%{_mandir}/man1/${file}.1" "%{buildroot}%{_mandir}/man1/${file}%{major_version}.1"
done
install -D %{SOURCE1} %{buildroot}%{_sysconfdir}/rpm/macros.lua

# create pkg-config file
cat > lua.pc <<-EOF
prefix=%{_prefix}
exec_prefix=%{_prefix}
libdir=%{_libdir}
includedir=%{_prefix}}/include

Name: Lua
Description: An Extensible Extension Language
Version: %{version}
Libs: -llua -lm
Cflags:
EOF
install -D -m 644 lua.pc %{buildroot}/%{_libdir}/pkgconfig/lua.pc

%check
cd src
LD_LIBRARY_PATH=`pwd` ./lua -e 'print(0)' > /dev/null

%post
/usr/sbin/update-alternatives --install                                                   \
            %{_bindir}/lua            lua       %{_bindir}/lua%{major_version}         15 \
    --slave %{_bindir}/luac           luac      %{_bindir}/luac%{major_version}           \
    --slave %{_mandir}/man1/lua.1.gz  lua.1.gz  %{_mandir}/man1/lua%{major_version}.1.gz  \
    --slave %{_mandir}/man1/luac.1.gz luac.1.gz %{_mandir}/man1/luac%{major_version}.1.gz

%preun
if [ "$1" = 0 ] ; then
    /usr/sbin/update-alternatives --remove lua %{_bindir}/lua%{major_version}
fi

%post -n liblua%{libsuf} -p /sbin/ldconfig

%postun -n liblua%{libsuf} -p /sbin/ldconfig

%files
%defattr(-,root,root)
%doc README
%ghost %{_mandir}/man1/lua.1.gz
%{_mandir}/man1/lua%{major_version}.1*
%ghost %{_mandir}/man1/luac.1.gz
%{_mandir}/man1/luac%{major_version}.1*
%ghost %{_bindir}/lua
%{_bindir}/lua%{major_version}
%ghost %{_bindir}/luac
%{_bindir}/luac%{major_version}
%dir %{_libdir}/lua
%dir %{_libdir}/lua/%{major_version}
%dir %{_datadir}/lua
%dir %{_datadir}/lua/%{major_version}
%{_sysconfdir}/rpm/macros.lua
%ghost %{_sysconfdir}/alternatives/luac
%ghost %{_sysconfdir}/alternatives/lua.1.gz
%ghost %{_sysconfdir}/alternatives/luac.1.gz
%ghost %{_sysconfdir}/alternatives/lua

%files -n liblua%{libsuf}
%defattr(-,root,root)
%{_libdir}/liblua.so.%{major_version}

%files devel
%defattr(-,root,root)
%{_includedir}/lauxlib.h
%{_includedir}/lua.h
%{_includedir}/lua.hpp
%{_includedir}/luaconf.h
%{_includedir}/lualib.h
%{_libdir}/liblua.a
%{_libdir}/liblua.so
%{_libdir}/pkgconfig/lua.pc

%files doc
%defattr(-,root,root)
%doc doc/*

%changelog
* Wed Sep 10 2014 pgajdos@suse.com
- CVE-2014-5461 [bnc#893824]
- added patches:
  * lua-CVE-2014-5461.patch
* Mon Sep  9 2013 pgajdos@suse.com
- updated to 5.2.2: bugfix release, fixed:
  * Stack overflow in vararg functions with many fixed parameters
    called with few arguments.
  * Garbage collector can trigger too many times in recursive loops.
  * Wrong assert when reporting concatenation errors (manifests only
    when Lua is compiled in debug mode).
  * Wrong error message in some short-cut expressions.
  * luac listings choke on long strings.
  * see http://www.lua.org/bugs.html for details
* Tue Feb 19 2013 pgajdos@suse.com
- fix visibility of global variables [bnc#803791]
  * modified lua-suse.diff
- added LUA_SO target as dependency to LUAC_T and LUA_T to fix
  parallel build
  * modified lua-suse.diff
- added regresion check for this error into %%check
* Tue Feb  5 2013 coolo@suse.com
- update alternatives need to be %%ghosts
* Thu Nov 15 2012 pgajdos@suse.com
- export luaU_dump in order to fix linking of luac binary
* Tue Oct  9 2012 crrodriguez@opensuse.org
- Tweak lua-suse.diff again:
  * LUA_IFUNC must be defined as upstream but without "extern",
  revert the previous mod.
  * LUA_API must be defined as visibility default
* Thu Aug 16 2012 crrodriguez@opensuse.org
- Even after the previous update, third party modules/libraries
  would not work because LUA_CDIR constant is wrong in x86_64
* Wed Aug 15 2012 crrodriguez@opensuse.org
- devel package must require libluaver = %%{version}
* Wed Aug 15 2012 crrodriguez@opensuse.org
- Broken lua-suse patch breaks loading third party modules
  or scripts
* Mon Jun 18 2012 pgajdos@suse.com
- updated to 5.2.1:
  * main thread predefined in the registry
  * bugfixes
* Thu Mar 29 2012 joop.boonen@opensuse.org
- Correction in Provides and Obsolete for lua-libs
- Passed the spec file through spec-cleaner
* Wed Jan 18 2012 dimstar@opensuse.org
- Require liblua5_2 from the -devel package: Otherwise the .so file
  is a dangling symlink and apps will use the static lib.
- Update baselibs.conf to reference liblua5_2 instead of liblua5_1.
* Fri Jan  6 2012 dmueller@suse.de
- readd lua.pc for building rpm
* Wed Dec 21 2011 pgajdos@suse.com
- raise update-alternatives priority to 15 (lua51 have 10, lua50
  have 5)
* Wed Dec 21 2011 pgajdos@suse.com
- updated to 5.2.0:
  * Main changes
  yieldable pcall and metamethods
  new lexical scheme for globals
  ephemeron tables
  new library for bitwise operations
  light C functions
  emergency garbage collector
  [goto] statement
  finalizers for tables
  * Language
  no more fenv for threads or functions
  tables honor the [__len] metamethod
  hex and [\z] escapes in strings
  support for hexadecimal floats
  order metamethods work for different types
  no more verification of opcode consistency
  hook event "tail return" replaced by "tail call"
  empty statement
  [break] statement may appear in the middle of a block
  * Libraries
  arguments for function called through [xpcall]
  optional 'mode' argument to load and loadfile (to control binary x text)
  optional 'env' argument to load and loadfile (environment for loaded chunk)
  [loadlib] may load libraries with global names (RTLD_GLOBAL)
  new function [package.searchpath]
  modules receive their paths when loaded
  optional base in [math.log]
  optional separator in [string.rep]
  [file:write] returns <CODE>file</CODE>
  closing a pipe returns exit status
  [os.exit] may close state
  new metamethods [__pairs] and <CODE>__ipairs</CODE>
  new option 'isrunning' for [collectgarbage] and <CODE>lua_gc</CODE>
  frontier patterns
  [\0] in patterns
  new option [*L] for <CODE>io.read</CODE>
  options for [io.lines]
  [debug.getlocal] can access function varargs
  * C API
  new functions: lua_absindex, lua_arith, lua_compare, lua_copy, lua_len,
    lua_rawgetp, lua_rawsetp, lua_upvalueid, lua_upvaluejoin,
  lua_version, luaL_checkversion, luaL_setmetatable,
  luaL_testudata, luaL_tolstring
    [lua_pushstring] and <CODE>pushlstring</CODE> return string
    [nparams] and <CODE>isvararg</CODE> available in debug API
  new lua_Unsigned
  * Implementation
  max constants per function raised to 2^26
  generational mode for garbage collection (experimental)
  NaN trick (experimental)
  internal (immutable) version of ctypes
  simpler implementation for string buffers
  parser uses much less C-stack space (no more auto arrays)
  * Lua standalone interpreter
  new [-E] option to avoid environment variables
  handling of non-string error messages
* Tue Oct  4 2011 uli@suse.com
- cross-build fix: use %%__cc macro
* Mon Jun 28 2010 jengelh@medozas.de
- use %%_smp_mflags
* Mon Jun 14 2010 pgajdos@suse.cz
- removed -L/usr/lib from Libs: in etc/lua.pc [bnc#613497]
- used upstream bugfix patch from http://www.lua.org/download.html
* Sat Apr 24 2010 coolo@novell.com
- buildrequire pkg-config to fix provides
* Tue Mar 23 2010 pgajdos@suse.cz
- modified LUA_PATH_DEFAULT [bnc#589441]: search modules under
  /usr/%%{_lib}/lua/%%{major_version} too
* Sat Dec 12 2009 jengelh@medozas.de
- add baselibs.conf as a source
- package documentation as noarch
* Sun Dec  6 2009 jengelh@medozas.de
- enable parallel building
* Tue Nov  3 2009 coolo@novell.com
- updated patches to apply with fuzz=0
* Wed Sep 16 2009 dmueller@suse.de
- add baselibs.conf
* Mon Jun  8 2009 pgajdos@suse.cz
- package owns /usr/{lib,share}/lua directories from now
- emoved unneeded root.patch
* Thu Sep 11 2008 pgajdos@suse.cz
- updated to 5.1.4: bugfix release
* Wed Jun 11 2008 jfunk@funktronics.ca
- Set LUA_ROOT to /usr and fix INSTALL_CMOD for lib64
- Add RPM macros file
- Allows for lua-<modulename> packages like python, perl, and ruby
* Fri Mar 28 2008 coolo@suse.de
- hint the solver about the package rename
* Wed Mar 26 2008 coolo@suse.de
- do not package/provide the shared lib twice / follow shared lib
  policy
* Tue Feb 19 2008 pgajdos@suse.cz
- created -doc package
- updated to version 5.1.3, bug fix release. Fixed bugs:
  * wrong error message in some concatenations
  * too many variables in an assignment may cause a C stack
    overflow
  * an error in a module loaded through the '-l' option shows no
    traceback
  * gsub may go wild when wrongly called without its third
    argument and with a large subject
  * table.remove removes last element of a table when given
    an out-of-bound index
  * lua_setfenv may crash if called over an invalid object
  * stand-alone interpreter shows incorrect error message when
    the "message" is a coroutine.
  * debug.sethook/gethook may overflow the thread's stack
* Tue Jul 24 2007 pgajdos@suse.cz
- using /sbin/ldconfig in specfile
- updated to version 5.1.2, fixes some bugs:
  * count hook may be called without being set
  * code generated for "-nil", "-true", and "-false" is wrong
  * recursive coroutines may overflow C stack
  * wrong error message in some concatenations
  * very small numbers all collide in the hash function
* Mon Nov 27 2006 mrueckert@suse.de
- package liblua.so as symlink.
* Thu Nov 16 2006 mrueckert@suse.de
- the core package now uses update alternatives to allow installation
  of 5.0.x and 5.1.x
- devel packages will conflict now
* Tue Nov 14 2006 mrueckert@suse.de
- ARGH dont copy and paste!:
  lua-libs should not depend on lua. another run on (#217875)
* Tue Nov 14 2006 mrueckert@suse.de
- really fix the file pattern (#217875)
* Thu Nov  9 2006 mrueckert@suse.de
- splitted out lua-libs so we can install at least both libs
  in parallel. (#217875)
* Thu Oct 19 2006 dmueller@suse.de
- fix wrong prefix in lua.pc
- link liblua with -lm to satisfy unresolved symbols
* Thu Oct 19 2006 mjancar@suse.cz
- include lua.pc
* Wed Oct 18 2006 mjancar@suse.cz
- update to 5.1.1
  Language:
  * new module system.
  * new semantics for control variables of fors.
  * new semantics for setn/getn.
  * new syntax/semantics for varargs.
  * new long strings and comments.
  * new `mod' operator (`%%')
  * new length operator #t
  * metatables for all types
  API:
  * new functions: lua_createtable, lua_get(set)field, lua_push(to)integer.
  * user supplies memory allocator (lua_open becomes lua_newstate).
  * luaopen_* functionst must be called through Lua.
  Implementation:
  * new configuration scheme via luaconf.h.
  * incremental garbage collection.
  * better handling of end-of-line in the lexer.
  * fully reentrant parser (new Lua function `load')
  * better support for 64-bit machines.
  * native loadlib support for Mac OS X.
  * standard distribution in only one library (lualib.a merged into lua.a)
* Wed Jan 25 2006 mls@suse.de
- converted neededforbuild to BuildRequires
* Sat Jan 14 2006 schwab@suse.de
- Don't strip binaries.
- Fix shared library linking.
* Thu Sep  8 2005 anicka@suse.cz
- enable dynamic loading capability (#106845)
* Fri Apr 23 2004 mcihar@suse.cz
- initial packaging
